library ieee;
use ieee.std_logic_1164.all;

entity datapath_tb is
end datapath_tb;

architecture rtl of datapath_tb is

    component datapath 
        port(
            clk: in std_logic;
            reset: in std_logic;
            memtoreg: std_logic;
            pcsrc: std_logic;
            alusrc: in std_logic;
            regdst: in std_logic;
            regwrite: in std_logic;
            jump: in std_logic;
            alucontrol: in std_logic_vector(2 downto 0);
            readdata: in std_logic_vector(31 downto 0);
            instr: in std_logic_vector(31 downto 0);
            zero: out std_logic;
            pc: buffer std_logic_vector(31 downto 0);
            aluout: buffer std_logic_vector(31 downto 0);
            writedata: buffer std_logic_vector(31 downto 0)
          );
        end component;

        --Inputs std_logic
        signal clk : std_logic := '1';
        signal reset : std_logic := '0';
        signal memtoreg : std_logic;
        signal pcsrc : std_logic;
        signal alusrc : std_logic;
        signal regdst : std_logic;
        signal regwrite : std_logic;
        signal jump : std_logic;
        
        --Outputs std_logic
        signal zero : std_logic;

        --Inputs std_logic_vector
        signal alucontrol: std_logic_vector(2 downto 0);
        signal readdata: std_logic_vector(31 downto 0);
        signal instr: std_logic_vector(31 downto 0);

        --Outputs std_logic_vector
        signal pc: std_logic_vector(31 downto 0);
        signal aluout: std_logic_vector(31 downto 0);
        signal writedata: std_logic_vector(31 downto 0);


        constant ClockFrequency : integer := 100e6; -- 100 MHz
        constant ClockPeriod    : time    := 2000 ms / ClockFrequency;

begin
    PROCESS BEGIN
        for i in 0 to 60 loop
        clk <= not clk after ClockPeriod / 2;
        wait for 10 ns;
        end loop;
        
        wait;
    END PROCESS;

    databpath_tb : datapath
    port map(
        clk  => clk,
        reset  => reset,
        memtoreg  => memtoreg,
        pcsrc  => pcsrc,
        alusrc => alusrc,
        regdst => regdst, 
        regwrite => regwrite, 
        jump => jump, 
        alucontrol => alucontrol,
        readdata => readdata,
        instr => instr, 
        zero => zero, 
        pc => pc, 
        aluout => aluout,
        writedata => writedata
);

    
    process begin  
        reset <= '1';
        wait for 20 ns;
        reset <= '0';
        wait for 20 ns;
        instr <= x"000F6950";           
        reset <= '0';          
        pcsrc <= '0';     
        readdata <= x"00000007";
        memtoreg <= '0';        
        alusrc <= '1';          
        regdst <= '0';          
        regwrite <= '1';      
        jump <= '0';
        alucontrol <= "010";   
        wait for 20 ns;
        wait;
        report "END";
    end process;
end rtl;